package be.iminds.ilabt.jfed.experimenter_gui.slice;

import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.lowlevel.GeniUser;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.rspec.model.BasicStringRspec;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import be.iminds.ilabt.jfed.rspec.rspec_source.ManifestRspecSource;
import be.iminds.ilabt.jfed.ssh_terminal_tool.ExternalSshTerminal;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;

import javax.annotation.Nonnull;
import javax.inject.Inject;
import javax.inject.Singleton;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

/**
 * User: twalcari
 * Date: 12/8/2014
 * Time: 14:00
 */
@Singleton
public class NodePropertiesDialogFactory {
    private final GeniUserProvider geniUserProvider;
    private final ExternalSshTerminal externalSshTerminal;

    @Inject
    public NodePropertiesDialogFactory(GeniUserProvider geniUserProvider, ExternalSshTerminal externalSshTerminal) {
        this.geniUserProvider = geniUserProvider;
        this.externalSshTerminal = externalSshTerminal;
    }


    public void showSimplePropertiesDialog(@Nonnull Slice slice, @Nonnull FXRspecNode node) {
        Optional<BasicStringRspec.LoginService> loginService = findLoginService(slice, node);
        if (loginService.isPresent()) {
            NodePropertiesDialog npd = new NodePropertiesDialog(slice, node, loginService.get(), externalSshTerminal);
            npd.showDialog();
        } else {
            JFDialogs.create()
                    .message("The selected node doesn't provide any information to perform an interactive login. Is this node ready?")
                    .masthead("No information available about this node")
                    .showWarning();
        }

    }


    private Optional<BasicStringRspec.LoginService> findLoginService(@Nonnull Slice slice, @Nonnull FXRspecNode node) {
        ManifestRspecSource manifestRspecSource = slice.getManifestRspec();
        List<BasicStringRspec.LoginService> loginServices = manifestRspecSource != null ?
                manifestRspecSource.getNodeLoginInfo(node) :
                null;

        if(loginServices == null)  {
            return Optional.empty();
        }

        //try to find a loginInfo with the username of the currently logged in user
        GeniUser geniUser = geniUserProvider.getLoggedInGeniUser();
        assert geniUser == null || geniUser.getUserUrn() != null;
        final String desiredUsername = geniUser != null ? geniUser.getUserUrn().getEncodedResourceName() : null;

        if (desiredUsername != null) {
            Optional<BasicStringRspec.LoginService> desiredLoginService =
                    loginServices.stream().filter(ls -> Objects.equals(desiredUsername, ls.getUsername())).findFirst();

            if (desiredLoginService.isPresent())
                return desiredLoginService;
        }

        //fallback: use the first loginService available
        return loginServices.stream().findFirst();
    }


}
